﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using System.Collections;

namespace CreateTextRes
{

    class Program
    {
        static void Main(string[] args)
        {
            int i = 0;
            int textResNum = 0;
            Hashtable fileNames = new Hashtable();
            String gameDirectory;
            String backupFileExt = ".CreateTextResBackup";
            bool preserveOrder = false;

            if (args.Length != 1)
            {
                Console.WriteLine("CreateTextRes");
                Console.WriteLine("Version 1.0 - 09/26/2012");
                Console.WriteLine("By Jeremiah Nellis");
                Console.WriteLine();
                Console.WriteLine("Puts Print() statements within SCI0 script files into text resources.");
                Console.WriteLine(@"Usage: CreateTextRes.exe <gameDir>");
                Console.WriteLine(@"       CreateTextRes.exe ""<gameDir>""");
                Environment.Exit(2); // Exit with error
            }

            gameDirectory = args[0];

            // Extract all the resources 
            Resources.Extract(Path.Combine(gameDirectory, "resource.001"), Path.Combine(gameDirectory, "extractedResources"), preserveOrder);

            // Parse the game.ini to find all the scripts
            string sourceDirectory = Path.Combine(gameDirectory, "src");
            bool scriptSection = false;

            foreach (string line in File.ReadLines(Path.Combine(gameDirectory, "game.ini")))
            {
                if (line.Contains("[") && line.Contains("]"))
                {
                    if (line.Contains("Script"))
                    {
                        scriptSection = true;
                    }
                    else
                    {
                        scriptSection = false;
                    }
                }

                if (scriptSection == true)
                {
                    if (line.Trim().Length > 0)
                    {
                        Match match = Regex.Match(line, @"n(\d+)=(\w+)", RegexOptions.IgnoreCase);
                        if (match.Success)
                        {
                            // add scriptnumber (key), scriptname (value) to hashtable
                            fileNames.Add(match.Groups[1].Value, match.Groups[2].Value + ".sc");
                        }
                    }

                }

            }

            // Loop through the script files, processing each one for Print statements
            foreach (DictionaryEntry fileName in fileNames)
            {
                bool foundPrintStmt = false;
                String textResFileName = null;

                textResNum = Convert.ToInt32(fileName.Key.ToString());
                i = 0;

                // Determine the starting textIndex number by checking to see if text resource already exists
                string[] files = Directory.GetFiles(Path.Combine(gameDirectory, "extractedResources"), 
                                                    "*TEXT." + textResNum.ToString().PadLeft(3,'0'), 
                                                    SearchOption.TopDirectoryOnly);
                if (files.Length > 0)
                {
                    int j;
                    // delete duplicate text resources
                    for (j = 0; j < files.Length - 2; j++)
                    {
                        File.Delete(files[j]);
                    }

                    // last in wins
                    textResFileName = files[files.Length - 1];

                    // read the file in & count the number of existing text resource entries
                    string existingTextRes = File.ReadAllText(textResFileName);
                    i = i + Regex.Matches(existingTextRes, "\x00").Count - 1;  // subtract 1 to account for text header
                }

                string input = File.ReadAllText(Path.Combine(sourceDirectory, fileName.Value.ToString()));
                List<String> textResLines = new List<String>();

                // single line print statements
                string pattern = @"Print\s*\(\s*""(.*?)""\s*\)";
                foreach (Match match in Regex.Matches(input, pattern))
                {
                    string replace = @"Print(" + textResNum + " " + i + ") /* " + match.Groups[1].Value + " */";
                    input = Regex.Replace(input, Regex.Escape(match.Groups[0].Value), replace);
                    textResLines.Add(match.Groups[1].Value);
                    i++;
                    foundPrintStmt = true;
                }

                // multi-line print statements
                pattern = @"Print\s*\((\s*""(.*?)""\s*\+\s*\n)+\s*""(.*?)""\s*\)";
                foreach (Match match in Regex.Matches(input, pattern))
                {
                    string multiLineStmt = "";
                    for (int captureCtr = 0; captureCtr < match.Groups[2].Captures.Count; captureCtr++)
                    {
                        multiLineStmt = multiLineStmt + match.Groups[2].Captures[captureCtr].Value;
                    }
                    multiLineStmt = multiLineStmt + match.Groups[3].Value;

                    string replace = @"Print(" + textResNum + " " + i + ") /* " + multiLineStmt + " */";
                    input = Regex.Replace(input, Regex.Escape(match.Groups[0].Value), replace);
                    textResLines.Add(multiLineStmt);
                    i++;
                    foundPrintStmt = true;
                }

                if (foundPrintStmt)
                {

                    string fileExt = Path.GetExtension(fileName.Value.ToString());
                    string fileNameNoExt = Path.GetFileNameWithoutExtension(fileName.Value.ToString());

                    // delete the backup script file if it exists
                    if (File.Exists(Path.Combine(sourceDirectory, fileNameNoExt + backupFileExt)))
                    {
                        File.Delete(Path.Combine(sourceDirectory, fileNameNoExt + backupFileExt));
                    }

                    // copy the original script file to the backup
                    File.Copy(Path.Combine(sourceDirectory, fileName.Value.ToString()), Path.Combine(sourceDirectory, fileNameNoExt + backupFileExt));

                    FileStream textRes = null;
  
                    if (textResFileName == null)
                    {
                        // create a new text resource file
                        if(preserveOrder) {
                            textRes = new FileStream(Path.Combine(Path.Combine(gameDirectory, "extractedResources"), "999_TEXT." + textResNum.ToString().PadLeft(3, '0')),
                                                     FileMode.Create, FileAccess.Write);

                        } else {
                            textRes = new FileStream(Path.Combine(Path.Combine(gameDirectory, "extractedResources"), "TEXT." + textResNum.ToString().PadLeft(3,'0')),
                                                     FileMode.Create, FileAccess.Write);
                        }

                        // write out the header
                        textRes.Write(HexUtilities.HexStringToBytes("8300"), 0, 2);
                    }
                    else
                    {
                        // open the existing text resource file
                        textRes = new FileStream(textResFileName, FileMode.Append, FileAccess.Write);
                    }

                    foreach (String line in textResLines)
                    {

                        byte[] bytes = Encoding.ASCII.GetBytes(line);
                        textRes.Write(bytes, 0, bytes.Length);

                        // terminate the text res entry
                        textRes.Write(HexUtilities.HexStringToBytes("00"), 0, 1);
                    }

                    textRes.Close();

                    // write out new the new script file
                    File.WriteAllText(Path.Combine(sourceDirectory, fileName.Value.ToString()), input);
                }

                Console.WriteLine("Processed file: {0}", fileName.Value.ToString());
            }  // end loop on script files
 
            // create a new resource.001 file
            Resources.Pack(Path.Combine(gameDirectory, "extractedResources"), Path.Combine(gameDirectory, "resource.001"));

            // create a new map file
            Resources.CreateMapFile(Path.Combine(gameDirectory, "resource.001"), Path.Combine(gameDirectory, "resource.map"));

            Console.WriteLine("Finished!");

            //Console.WriteLine();
            //Console.WriteLine("<Press Enter to exit>");
            //Console.ReadLine();
        }

    }
}

